/*
 * BatchWindow.cpp
 *
 * Written by Matt Pauker
 */

#ifndef BATCH_WINDOW_H
#include "BatchWindow.h"
#endif

#ifndef CHANGE_CONSTANTS_H
#include "ChangeConstants.h"
#endif

B_DEFINE_CLASS_INFO(BatchWindow, ChangeWindow);

BatchWindow::BatchWindow(long topX, long topY, LockableList *refs) :
	ChangeWindow(topX, topY)	
{
	SetTitle("Batch Change");
	
	refList = refs;
	
	// Add files menu
	BMenu *myMenu;
	BMenuItem *myMenuItem;
	
	myMenu = new BMenu("Files", B_ITEMS_IN_COLUMN);
	menuBar->AddItem(myMenu);
	
	long x, numItems;
	char *name = new char[B_FILE_NAME_LENGTH+1];
	BFile file;
	record_ref ref;
	
	refList->Lock();
	numItems = refList->CountItems();
	for (x = 0 ; x < numItems; x++)
	{
		ref = *((record_ref *)refList->ItemAt(x));
		file.SetRef(ref);
		file.GetName(name);
		
		myMenuItem = new BMenuItem(name, new BMessage(FILE_SELECTED), NULL, NULL);
		myMenuItem->SetMarked(TRUE);
		myMenu->AddItem(myMenuItem);
	}
	refList->Unlock();
	
	// Give focus to the type text box
	
	Lock();
	typeView->MakeFocus();
	Unlock();
	
}

void BatchWindow::Quit()
{
	long x, numItems;
	void *item;
	
	refList->Lock();
	numItems = refList->CountItems();
	for (x = numItems - 1; x >= 0; x--)
	{
		item = refList->RemoveItem(x);
		delete item;
	}
	refList->Unlock();
	
	delete refList;

	inherited::Quit();
}

bool BatchWindow::QuitRequested()
{
	if (be_app->CountWindows() == 1)		// If we're closing the only window open, quit
		be_app->PostMessage(B_QUIT_REQUESTED);
	return TRUE;
}

void BatchWindow::MessageReceived(BMessage *message)
{
	switch (message->what)
	{
		case FILE_SELECTED:
			BMenuItem *theItem;
			
			theItem = (BMenuItem *)message->FindObject("source");
			if (theItem->IsMarked())
				theItem->SetMarked(FALSE);
			else
				theItem->SetMarked(TRUE);
		break;
		default:
			inherited::MessageReceived(message);
		break;
	}
}