/*
 * SigView.cpp
 *
 * Written by Matt Pauker
 */

#ifndef SIG_VIEW_H
#include "SigView.h"
#endif

#ifndef INFO_WINDOW_H
#include "InfoWindow.h"
#endif

#ifndef CHANGE_CONSTANTS_H
#include "ChangeConstants.h"
#endif

B_DEFINE_CLASS_INFO(SignatureView, BTextView);

SignatureView::SignatureView(BRect frame, const char *name) :
				BTextView(frame, name, frame, B_FOLLOW_ALL, B_WILL_DRAW)
{
}

void SignatureView::AttachedToWindow()
{
	BTextView::AttachedToWindow();
	
	SetFontName("Kate");
	SetMaxChars(4);		// Signatures should only be 4 characters long

	BRect textRect;
	
	// Determine the textRect
	
	textRect = Bounds();
	textRect.OffsetTo(0, 0);
	textRect.InsetBy(1, 1);
	
	SetTextRect(textRect);

	DisallowChar(B_ENTER);	// We don't want the user entering an LF
}

bool SignatureView::AcceptsChar(ulong aChar) const
{
	if (aChar == B_TAB)
	{
		InfoWindow *myWindow;
		
		myWindow = (InfoWindow *)Window();
		myWindow->NextFocus();	// Tell the window to move the focus to the next box
		return FALSE;
	}
	else
		return BTextView::AcceptsChar(aChar);
}

void SignatureView::MessageReceived(BMessage *message)
{
	switch (message->what)
	{
		case SELECT_ALL:
			SelectAll();
		break;
		case B_DELETE:
			Delete();
		break;
		default:
			BTextView::MessageReceived(message);
		break;
	}
}

void SignatureView::SetSig(ulong code)
// Turns code into a string and places that string in the text view
{
	if (code > 0)
		SetText((char *)&code, 4);
	else
		SetText("", 0);
}

ulong SignatureView::GetSig()
// Gets the string in the text view and turns it into a ulong, so that it can be saved
{
	if (TextLength() > 0)
		return * (ulong *)Text();
	else
		return 0;
}

void SignatureView::KeyDown(ulong aChar)
{
	long start, finish;
			
	GetSelection(&start, &finish);
	if ((TextLength() < 4) || (finish - start > 0))
	{
		bool changed;
	
		changed = TRUE;
		
		switch (aChar)
		{
			case B_BACKSPACE:
				if (start == 0 && finish == 0)
					changed = FALSE;
			break;
			
			case B_UP_ARROW:
			case B_LEFT_ARROW:
			case B_DOWN_ARROW:
			case B_RIGHT_ARROW:
			case B_HOME:
			case B_END:
			case B_PAGE_UP:
			case B_PAGE_DOWN:
				changed = FALSE;
			break;
		}
		
		if (changed)
			((InfoWindow *)Window())->SetChanged(TRUE);
	}	
	inherited::KeyDown(aChar);
}

